function sx = bSplineAuswertung(t, grad, P, xVektor)

% Angewandte Numerik 2, Wintersemester 2016/2017, 09.12.2016
% Uebungsblatt 07, Aufgabe 24: effiziente Auswertung von B-Splines
%
% wertet den Spline an den Stellen x (aus xVektor) aus
%
% Parameter
%   t:          Vektor t_0, ..., t_n: Die Knoten
%   grad:       Der Grad des Splines
%   P:          Die Kontrollpunkte P_0, ... P_l: Matrix aus R^(l+1 x d):
%               (Koeffizienten fuer die B-Splinebasisfktnen vom Grad grad)
%               Im Fall d==1: Vektor c_0, ..., c_l
%   xVektor:	Vektor x_1, ..., x_m der Stellen,
%               an denen der Spline ausgewertet werden soll
%
% Rueckgabewert:
%   sx:         Matrix aus R^(m x d): Vektor der Werte des Splines
%               Zu jedem Wert k = 1, ..., m aus xVektor ein Funktions-
%               Wert des Splines: sx_k = (sx_{k,1}, ..., sx_{k,d})



%% Initialisierungen

n  = length(t);              	% Anzahl der Knoten t_i
t  = t(:);                      % Spaltenvektor

d  = size(P, 2);                % Dimension des Problems
sx = zeros(length(xVektor), d);



%% Sonderfall: Alle Knoten sind gleich
%  also: Alle B-Spline-Basis-Funktionen haben den Wert 0
%  also: Alle Werte des B-Splines sind 0

if t(1+grad) == t(n-grad)
    return;                	% Beachte: sx ist Null-Vektor
end



%% B-Spline an allen Stellen x auswerten

for i = 1:length(xVektor)
    
    
    
    %% einen x-Wert und zugehoeriges Intervall bestimmen
    
    x = xVektor(i);
    
    if x < t(1+grad)
        error('x = %g muss >= kleinstem Knoten t = %g sein', x, t(1+grad));
    end

    if x > t(end-grad)
        error('x = %g muss <= groesstem Knoten t = %g sein', x, t(end-grad));
    end
    
    j = n-grad;
    if x == t(j)    	% Sonderfall: rechtes Intervall ist abgeschlossen
        while x == t(j) % deshalb rechte Intervallgrenze noch dazu nehmen
            j = j-1;
        end
        % hier gilt: t(j) < x == t(j+1)      (geht, da t(1) < t(end))
        % also insbesondere: x liegt im letzten, abgeschlossenen Intervall
    else                % hier ist also x < t(n-grad)
        while x < t(j)
            j = j-1;
        end
        % hier gilt: t(1+grad) <= t(j) <= x < t(j+1) und j >= 1+grad
    end
    % hier gilt: x liegt im Intervall [t(j), t(j+1)) bzw. [t(j), t(j+1)]

    jg = j - grad;
    
    
    
    %% Werte der B-Spline-Basisfunktionen an der Stelle x berechnen
    %  (Dreiecksschema durchfuehren)
    
    werteN     = zeros(n-grad-1, 1);
    werteN(jg) = 1;               	% Konstantes Basispolynom
                                    % im Intervall [t(j), t(j+1))
    for p = 1:grad
        faktor1         = (t(j+1:j+p) - x) ./ (t(j+1:j+p) - t(j-p+1:j));
        faktor2         = (x - t(j-p+1:j)) ./ (t(j+1:j+p) - t(j-p+1:j));
        ind             = find(t(j+1:j+p) == t(j-p+1:j));
        faktor1(ind)    = 0;        % hier haben wir durch 0 geteilt
        faktor2(ind)    = 0;       	% Diese Faktoren sind per Definition 0
        
        summand1        = [faktor1 .* werteN(jg:jg+p-1); 0];
        summand2        = [0; faktor2 .* werteN(jg:jg+p-1)];
        werteN(jg:jg+p) = summand1 + summand2;
    end
    
    if (size(werteN) ~= [n-grad-1, 1])
        error('Programmierfehler: size(werteN) ~= [n-grad-1, 1]');
    end
    
    
    
    %% B-Spline auswerten

    sx(i,:) = werteN' * P;
end
    



end

