% Angewandte Numerik 2, Wintersemester 2016/2017, 09.12.2016
% Uebungsblatt 07, Aufgabe 24: Effiziente Auswertung von B-Spline-Kurven
%
% Hauptprogramm
% - wertet den B-Spline aus Aufgabe 23 an der Stelle 1/2 aus
% - plottet den B-Spline aus Aufgabe 23
% - plottet alle B-Spline-Basisfunktionen bis zum Grad 3
%       zur Knotenfolge aus Aufgabe 23



clear all;
close all;
clc;



%% Initialisierungen: Aenderbar

maxGrad   = 3;                      % Grad, bis zu dem die B-Spline-
                                    % Basisfunktionen geplottet
                                    % werden sollen
anzXWerte = 200;                    % Feinheit der Diskretisierung

                                    

%% quadratischen B-Spline aus Aufgabe 9 an einer Stelle auswerten

grad = 2;
ti   = [0, 0, 0, 1, 2, 3, 3, 3];    % 8 Stuetzstellen
                                    % 7 B-Spline-Basisfkt. vom Grad 0
                                    % 6 B-Spline-Basisfkt. vom Grad 1
                                    % 5 B-Spline-Basisfkt. vom Grad 2
                                    
cj2  = [2; 2; 4; 2; 5];             % 5 Koeffizienten fuer
                                    % 5 B-Spline-Basisfkt. vom Grad 2
                                    
x    = 0.5;                         % Auswertestelle
sx   = bSplineAuswertung(ti, grad, cj2, x);     % Auswertung

fprintf('\n');
fprintf('Angewandte Numerik 2, Wintersemester 2016/2017, 09.12.2016\n');
fprintf('Uebungsblatt 07, Aufgabe 24:');
fprintf(' Effiziente Auswertung von B-Spline-Kurven\n');
fprintf('\n');

fprintf('Der interpolierende quadratische B-Spline aus Aufgabe 23\n');
fprintf('hat an der Stelle x = %g den Wert %g.\n', x, sx);
fprintf('\n');



%% quadratischen B-Spline aus Aufgabe 9 plotten

xWerte  = linspace(0, 3, anzXWerte);        % Diskretisierung x-Achse
sxWerte = bSplineAuswertung(ti, grad, cj2, xWerte);

figure;
plot (xWerte, sxWerte, '-*')
titleStr = sprintf('B-Spline vom Grad %d aus Aufgabe 23', grad);
titleStr = sprintf('%s zum Knotenvektor (%g', titleStr, ti(1));
for i = 2:length(ti)
    titleStr = sprintf('%s, %g', titleStr, ti(i));
end
titleStr = sprintf('%s)', titleStr);
title(titleStr);
xlabel('x');
ylabel('s(x)');



%% alle B-Spline-Basisfunktionen vom Grad 0 bis maxGrad plotten

t      = [0, 1, 2, 3];                        	% 4 Stuetzstellen
xWerte = linspace(t(1), t(end), anzXWerte);     % Diskretisierung x-Achse

for grad = 0:maxGrad

    ti      = [t(1)*ones(1,grad), t, t(end)*ones(1,grad)];
    % (minimale) Knotenfolge fuer B-Splines vom Grad grad
    % zur Tabelle aus Aufgabe 23
    % Beachte:
    %   B-Spline-Basisfunktionen, die identisch 0 sind,
    %   werden nicht geplottet.
    %   In der Musterloesung zu Aufgabe 25 (bSplineBasisfunktionen.m)
    %   werden auch diese geplottet.
    
    lenTi   = length(ti);               % Anzahl der Knoten
    C       = eye(lenTi-grad-1);
    % Jede Zeile ist ein Koeffizientenvektor,
    % jede Zeile liefert eine B-Spline-Bsisfunktion vom Grad grad

    sxWerte = zeros(length(xWerte), lenTi-1);
    % Funktionswerte der B-Spline-Basisfunktionen:
    % Fuer jeden diskreten x-Wert eine Zeile,
    % fuer jede B-Spline-Basisfunktion eine Spalte
    
    legendStr = cell(lenTi-grad-1, 1);
    % Strings fuer die Legende, fuer jedes Schaubild neu
    
    for i = 1:lenTi-grad-1
        sxWerte(:,i) = bSplineAuswertung(ti, grad, C(:,i), xWerte);
        legendStr{i} = sprintf('N_{%d}^{%d}', i-1, grad);
    end

    figure;
    
    plot(xWerte, sxWerte)
  	hold on
    plot([t(1)-0.2, t(end)+0.5], [0, 0], 'k');  % x-Achse
    plot([0, 0], [-0.2, 1.2], 'k');             % y-Achse
    
    xlabel('x');
    ylabelStr = sprintf('N_i^%d(x)', grad);
    ylabel(ylabelStr);
    titleStr = 'B-Spline-Basisfunktionen vom Grad';
    titleStr = sprintf('%s %d zum Knotenvektor (', titleStr, grad);
    titleStr = sprintf('%s%g', titleStr, ti(1));
    for i = 2:length(ti)
        titleStr = sprintf('%s, %g', titleStr, ti(i));
    end
    titleStr = sprintf('%s)', titleStr);
    title(titleStr);
    axis([t(1)-0.2, t(end)+0.5, -0.2, 1.2])
    legend(legendStr)
    hold off
end
